#!/bin/bash
# A simplified (and somewhat corrected) version of Mdk's ide-devfs.sh

KERNELNAME=$1
BUSID=$2
PARTNUM=$3
DRIVETYPE=$4

[ -s /etc/udev/udev.conf ] && . /etc/udev/udev.conf

: ${udev_root:=/dev}
udev_root=${udev_root%/}

: ${DRIVETYPE:=ide}

function failback() {
	# fail back case. Maybe I should simply die at this
	# point and try to success w/ another rule
# 	echo $KERNELNAME 
# 	exit 0
	exit 1
}

if [ -z "$PARTNUM" ]; then
	# Need to understand drives origin: a disk, a cdrom or an ide-floppy
	if [ "x$DRIVETYPE" = "xide" ]; then
		[ ! -r /proc/ide/${KERNELNAME}/media ] && failback
		MEDIA=`cat /proc/ide/${KERNELNAME}/media`
		case "x$MEDIA" in
			xcdrom)
			 [ -x /etc/udev/scripts/cd-names.sh ] && exec /etc/udev/scripts/cd-names.sh $KERNELNAME
			 # Fallback case
			 echo -n "cdroms/cdrom%N"
			 #[ -L $udev_root/cdrom ] || echo -n " cdrom"
			 echo
			;;
			xdisk)
			 echo "discs/disc%N/disc"
			;;
			xfloppy)
			 echo "floppies/floppy%N"
			;;
			*)
			 failback 1
			;;
		esac
	elif [ "x$DRIVETYPE" = "xscsi" ]; then
		case "x$KERNELNAME" in
			xsd[a-z]*)
			 echo "discs/disc%N/disc"
                        ;;
			*)
			 failback 1
			;;
		esac
	else
	    failback 6
	fi
else
	# A harder case. We need to understand what "disc" label was assigned
	# to the whole device. The problem is also in parallel delivering of
	# device creation events even for drives w/ partitions. I'm not sure
	# that is not a bug of udev :-/
	driveName=${KERNELNAME%%[0-9]*}

	# polling sucks badly
	timeout=5

	while test $timeout -gt 0 -a ! -e $udev_root/$driveName ; do
	    timeout=$(($timeout-1))
	    sleep 1
	done

	# Several cases are possible
	# 1. /dev/$driveName is a link to a block device that looks like
	# discs/*/disc  (i.e. shortened devfs scheme is in use).
	#   => we should report partition as part$PARTNUM in correspondent
	# discs/*/ subdirectory, which should be retrived by readlink
	# and some bash magic.
	
	# 2. /dev/$driveName is a block device itself. I assume that the
	# same script was used to name the whole drive, so some sort of
	# /dev/discs/*/disc was already created and probably it's a symlink
	# to /dev/$driveName. Fine. We're going to see which entry points
	# to /dev/$driveName and report partition for it.
	#
	# 3. /dev/$driveName is a link but either not to a block device or
	# to a block device outside /dev/discs. We can't do anything w/
	# that and th entire rule should be marked as failed
	#
	# 4. /dev/$driveName is a block device but appropriate
	# /dev/discs/*/disc can't be found. Also the rule is failed
	# 5. /dev/$driveName is neither a link nor a block. The rule is failed

	if [ -L $udev_root/$driveName ]; then
	    [ -b $udev_root/$driveName ] || failback 2
	    disc=`readlink $udev_root/$driveName` || failback 3
	    [ "x${disc#discs/}" = "x$disc" -o "x${disc%/disc}" = "x$disc" ] && failback 3
	    echo ${disc%disc}part$PARTNUM
	elif [ -b $udev_root/$driveName  ]; then
	    for disc in $udev_root/discs/*/disc; do
		[ -L $disc ] || continue
		drive=`realpath $disc` || continue
		[ "x$drive" = "x$udev_root/$driveName" ] || continue
		echo ${disc%disc}part$PARTNUM
		exit 0
	    done
	    # No appropriate link
	    failback 4
	else
	    failback 5
        fi
fi
exit 0

